require("defines")

-- Initializes global table.
script.on_init(function()
	if not global.supportive_turrets then global.supportive_turrets = {} end
end)


-- Creates a Turret 1 Unit at the same position of the given Turret 1.
local function build_unit_for_turret_1(turret_1_entity)
	-- Make sure Turret 1 has not been registered.
	local surface = turret_1_entity.surface
	local position = turret_1_entity.position
	if global.supportive_turrets[surface.name] then
		if global.supportive_turrets[surface.name][position.x] then
			if global.supportive_turrets[surface.name][position.x][position.y] then
				return
			end
		end
	end
	
	-- Create unit.
	local surface = turret_1_entity.surface
	local unit = surface.create_entity
	{
		name = supportive_turrets_defines.names.entities.supportive_turret_1_unit,
		position = {x = position.x - 0.01, y = position.y},
		force = turret_1_entity.force,
		direction = turret_1_entity.direction,
		target = turret_1_entity
	}
	-- Make the unit indestructible.
	if unit then
		unit.destructible = false
		
		-- Order the unit to attack Turret 1.
		unit.set_command
		{
			type = defines.command.attack,
			target = turret_1_entity,
			distraction = defines.distraction.none
		}
	end
	
	-- Register Turret 1 with Turret 2.
	if not global.supportive_turrets[surface.name] then global.supportive_turrets[surface.name] = {} end
	if not global.supportive_turrets[surface.name][position.x] then global.supportive_turrets[surface.name][position.x] = {} end
	global.supportive_turrets[surface.name][position.x][position.y] =
	{
		turret = turret_1_entity,
		unit = unit
	}
end

-- Removes the unit of given Turret 1.
local function remove_unit_from_turret_1(turret_1_entity)
	-- Make sure Turret 1 has been registered.
	local surface = turret_1_entity.surface
	local position = turret_1_entity.position
	if not global.supportive_turrets[surface.name] then
		return
	end
	if not global.supportive_turrets[surface.name][position.x] then
		return
	end
	if not global.supportive_turrets[surface.name][position.x][position.y] then
		return
	end
	local unit = global.supportive_turrets[surface.name][position.x][position.y].unit
	if unit then
		unit.destroy()
	end
	global.supportive_turrets[surface.name][position.x][position.y] = nil
end



-- Creates unit when a Turret 1 is built, and records them.
script.on_event(defines.events.on_built_entity, function(event)
	if event.created_entity.name == supportive_turrets_defines.names.entities.supportive_turret_1 then
		build_unit_for_turret_1(event.created_entity)
	end
end)

-- Creates unit when a Turret 1 is built by robot.
script.on_event(defines.events.on_robot_built_entity, function(event)
	if event.created_entity.name == supportive_turrets_defines.names.entities.supportive_turret_1 then
		build_unit_for_turret_1(event.created_entity)
	end
end)

-- Creates unit when a Turret 1 is canceled for deconstruction.
script.on_event(defines.events.on_canceled_deconstruction, function(event)
	if event.entity.name == supportive_turrets_defines.names.entities.supportive_turret_1 then
		build_unit_for_turret_1(event.entity)
	end
end)




-- Removes the associated unit when a Turret 1 is mined.
script.on_event(defines.events.on_preplayer_mined_item, function(event)
	if event.entity.name == supportive_turrets_defines.names.entities.supportive_turret_1 then
		remove_unit_from_turret_1(event.entity)
	end
end)

-- Removes the associated unit when a Turret 1 is killed.
script.on_event(defines.events.on_entity_died, function(event)
	if event.entity.name == supportive_turrets_defines.names.entities.supportive_turret_1 then
		remove_unit_from_turret_1(event.entity)
	end
end)

-- Removes the associated unit when a Turret 1 is marked for deconstruction.
script.on_event(defines.events.on_marked_for_deconstruction, function(event)
	if event.entity.name == supportive_turrets_defines.names.entities.supportive_turret_1 then
		remove_unit_from_turret_1(event.entity)
	end
end)


-- We don't have any event for detecting when Turret 1 is removed by LuaEntity::destroy().
-- on_tick is the only solution for now.
local function on_tick(event)
	for surface_name, surface_position_x in pairs(global.supportive_turrets) do
		for x, surface_position_y in pairs(surface_position_x) do
			for y, data in pairs(surface_position_y) do
				if not data.turret.valid then
					if data.unit.valid then
						data.unit.destroy()
					end
					global.supportive_turrets[surface_name][x][y] = nil
				end
			end
		end
	end
end

-- Remote interface.
remote.add_interface("supportive-turrets", 
{
	add_on_tick = function() script.on_event(defines.events.on_tick, on_tick) end,
	
	remove_on_tick = function() script.on_event(defines.events.on_tick, nil) end
})